/*******************************************************/
/*  I2C.C:                                             */
/*  Unit mit dem I2C-Bus-Anschluss als Master          */
/*  SCL = PTE19                                        */
/*  SDA = PTE18                                        */
/*******************************************************/

#include "stdtypes.h"
#include "MK10DN.h"
#include "i2c.h"


#define dSCL      (1<<19)
#define dSDA      (1<<18)

void Init_I2C (void)
{ PORTE_PCR18 =  (1<<8);           // I2C0_SDA als schnder GPIO
  PORTE_PCR19 =  (1<<8);           // I2C0_SCL als schnder GPIO
  GPIOE_PDDR  &= ~(dSCL | dSCL);   // Dir auf IN
  GPIOE_PCOR  =  dSCL | dSDA;      // beide Bits intern auf low
}

void SCL_low (void)                // Bit auf low
{ GPIOE_PCOR  = dSCL;
  GPIOE_PDDR |= dSCL;
}

void SCL_high (void)               // Bit auf high
{ GPIOE_PDDR &= ~dSCL; }

void SDA_low (void)
{ GPIOE_PCOR  = dSDA;
  GPIOE_PDDR |= dSDA;
}

void SDA_high (void)
{ GPIOE_PDDR &= ~dSDA; }


/*
Startbedingung: whrend SCL=hi wechselt SDA von hi auf lo
Stopbedingung:  whrend SCL=hi wechselt SDA von lo auf hi

Alle Byte-bertragungen starten und enden mit SCL=lo und SDA=hi.
Somit mssen die Vorbedingungen und die Nachbereitung
vom Do_I2C_Start /_Stop erledigt werden
(SDA=hi weil danach sowieso ACK kommt)
*/



void sdelay (long L) { while (L) --L; }

byte I2C_dio (byte sende)
{ byte maske, lese;
  lese = 0;
  maske = 0x80;
  while (maske)
  { if (sende & maske) SDA_high(); else SDA_low();
    sdelay(40);
    SCL_high();
    sdelay(80);
    if (GPIOE_PDIR & dSDA)  lese = lese | maske;
    SCL_low();
    sdelay(40);
    maske = maske>>1;
  }
  SDA_high();
  return lese;
}


/* bertragung starten und adressieren */
bool Do_I2C_Start (byte Adresse, byte R_W)
{ bool e;
  byte b;
  SCL_high();           /* fr die Start-Cond. mu SCL hi sein */
  sdelay(100);
  b = Adresse * 2;
  if (R_W) ++b;
  SDA_low();            /* Start-Cond. */
  sdelay(100);
  SCL_low();
  sdelay(50);           /* so, beides low, Bytebertragung kann losgehen */

  I2C_dio(b);

  SDA_high();           /* fertigmachen zum ACK einholen */
  sdelay(30);
  SCL_high();           /* Clockpuls fr ACK */
  sdelay(100);
  e = true;
  if (GPIOE_PDIR  & dSDA) e = false;
  SCL_low();
  sdelay(150);
  /* SCL jetzt lo, SDA sollte hi sein */
  return e;
}


/* ein Byte schreiben */
bool Do_I2C_Write (byte aByte)
{ bool e;
  I2C_dio(aByte);
  sdelay(500);
  SCL_high();
  sdelay(3000);
  e = true;
  if (GPIOE_PDIR & dSDA) e = false;
  SCL_low();
  SDA_low();
  sdelay(500);
  return e;
}


/* ein Byte lesen */
byte Do_I2C_Read   (bool Acknowledge)
{ byte e;
  e = I2C_dio(0xFF);
  sdelay(50);
  if (Acknowledge)
    SDA_low();
  else
    SDA_high();
  sdelay(40);
  SCL_high();
  sdelay(80);
  SCL_low();
  sdelay(40);
  SDA_high();
  return e;
}


/* bertragung stoppen */
void Do_I2C_Stop (void)
{ SDA_low();
  sdelay(100);
  SCL_high();
  sdelay(100);
  SDA_high();
  sdelay(200);
}

